Package: notame
Type: Package
Title: Workflow for non-targeted LC-MS metabolic profiling
Version: 1.3.0
Authors@R: c(
    person("Anton", "Klåvus", role = c("aut", "cph"),
           comment = c(ORCID = "0000-0003-2612-0230")),
    person("Jussi", "Paananen", role = c("aut", "cph"),
           comment = c(ORCID = "0000-0001-5100-4907")),
    person("Oskari", "Timonen", role = c("aut", "cph"),
           comment = c(ORCID = "0000-0002-6317-6260")),
    person("Atte", "Lihtamo", role = "aut"),
    person("Vilhelm", "Suksi", email = "vksuks@utu.fi", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0005-1108-518X")),
    person("Retu", "Haikonen", role = "aut",
           comment = c(ORCID = "0000-0003-0830-3850")),
    person("Leo", "Lahti", role = "aut",
           comment = c(ORCID = "0000-0001-5537-637X")),
    person("Kati", "Hanhineva", role = "aut",
           comment = c(ORCID = "0000-0001-6834-7375")),
    person("Ville", "Koistinen", role = "ctb",
           comment = c(ORCID = "0000-0003-1587-8361")),
    person("Olli", "Kärkkäinen", role = "ctb",
           comment = c(ORCID = "0000-0003-0825-4956")),
    person("Artur", "Sannikov", role = "ctb",
           comment = c(ORCID = "0000-0001-7765-123X")))
Description: Provides functionality for untargeted LC-MS metabolomics
        research as specified in the associated protocol article in the
        'Metabolomics Data Processing and Data Analysis—Current Best
        Practices' special issue of the Metabolites journal (2020).
        This includes tabular data preprocessing and quality control,
        uni- and multivariate analysis as well as quality control
        visualizations, feature-wise visualizations and results
        visualizations. Raw data preprocessing and functionality
        related to biological context, such as pathway analysis, is not
        included.
License: MIT + file LICENSE
Encoding: UTF-8
biocViews: BiomedicalInformatics, Metabolomics, DataImport,
        MassSpectrometry, BatchEffect, MultipleComparison,
        Normalization, QualityControl, Visualization, Preprocessing
Depends: R (>= 4.5.0), ggplot2, SummarizedExperiment
Imports: BiocGenerics, BiocParallel, dplyr, futile.logger, methods,
        openxlsx, S4Vectors, scales, stringr, tidyr, utils
Suggests: BiocStyle, fpc, igraph, knitr, missForest, notameViz,
        notameStats, pcaMethods, RUVSeq, testthat
URL: https://github.com/hanhineva-lab/notame,
        https://hanhineva-lab.github.io/notame/
BugReports: https://github.com/hanhineva-lab/notame/issues
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/parallel: true
Config/pak/sysreqs: libicu-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 13:05:37 UTC
RemoteUrl: https://github.com/bioc/notame
RemoteRef: HEAD
RemoteSha: d7eaa1a10cbc791a51d28fabd0db77a942682bc1
NeedsCompilation: no
Packaged: 2026-05-22 09:52:51 UTC; root
Author: Anton Klåvus [aut, cph] (ORCID:
    <https://orcid.org/0000-0003-2612-0230>),
  Jussi Paananen [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-5100-4907>),
  Oskari Timonen [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-6317-6260>),
  Atte Lihtamo [aut],
  Vilhelm Suksi [aut, cre] (ORCID:
    <https://orcid.org/0009-0005-1108-518X>),
  Retu Haikonen [aut] (ORCID: <https://orcid.org/0000-0003-0830-3850>),
  Leo Lahti [aut] (ORCID: <https://orcid.org/0000-0001-5537-637X>),
  Kati Hanhineva [aut] (ORCID: <https://orcid.org/0000-0001-6834-7375>),
  Ville Koistinen [ctb] (ORCID: <https://orcid.org/0000-0003-1587-8361>),
  Olli Kärkkäinen [ctb] (ORCID: <https://orcid.org/0000-0003-0825-4956>),
  Artur Sannikov [ctb] (ORCID: <https://orcid.org/0000-0001-7765-123X>)
Maintainer: Vilhelm Suksi <vksuks@utu.fi>
Built: R 4.6.0; ; 2026-05-22 09:56:10 UTC; unix
