# Write SummarizedExperiment objects from data-raw/example_data.R to file
write_to_excel(toy_notame_set, file = "inst/extdata/toy_notame_set.xlsx")
write_to_excel(hilic_neg_sample, file = "inst/extdata/hilic_neg_sample.xlsx")
write_to_excel(hilic_pos_sample, file = "inst/extdata/hilic_pos_sample.xlsx")
write_to_excel(rp_neg_sample, file = "inst/extdata/rp_neg_sample.xlsx")
write_to_excel(rp_pos_sample, file = "inst/extdata/rp_pos_sample.xlsx")